//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : HandleDDS.c
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------

// Include the board file description
#include "Board.h"
	 
//#include "stdio.h"
#include <math.h>

#include <inttypes.h>

#include "display.h"
#include "menu.h"
#include "SPI_ctrl.h"
#include "eepromhandler.h"
#include "HandleDDS.h"

//++++++++++++++++++++++++++++++++++++++++++++

union 	
	{
		volatile BYTE    			AD9834Buffer [ 4 ];
		volatile UINT					FQbuf;
	} fqBuffer;
	
struct FQvalues RAM;

volatile short					Value2change;
volatile short					SumValue;
volatile 	UINT				FqStep = 1;
volatile BYTE 				IncreaseFQ = 1;

const char cTransverter [] 							= "Transvert.";

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SendDivisor2DDS ( const AT91PS_SPI		pSpiIn )
{
	unsigned long Data_To_Send;
	
	
	EnableDDS;
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
// Cfg SPI MR
		// SPI is in master mode + CS not variable + CS directly connected + 
		// no Fault mode + min DLYBCS
		pSpiIn -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_SPI_CS0 );
		// For device 1 ( AD9834 ) set speed to MCKL/16 and 8 bits, 
		// CS active after transfer, mode 1, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 8 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_8 | 
										AT91C_SPI_NCPHA | 	//Mode 2 
										AT91C_SPI_CPOL | 		//Mode 2 
										AT91C_SPI_CSAAT | 
                    ( 0 << 24 ), 
										CSR0 );
	
	Data_To_Send = MSBCONTROLBYTE | ( ADD_SPI_CS0 ); // reset not set
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );

  Data_To_Send = LSBCONTROLBYTE | ( ADD_SPI_CS0 ) | LASTXFER;										
  AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
  AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );

	Data_To_Send = MSBFQ_0_BYTEWMASK |
								 (( fqBuffer.FQbuf >> 8 ) & 0x3F ) | 
								 ( ADD_SPI_CS0 );	
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );

	Data_To_Send = (( fqBuffer.FQbuf ) & 0xFF ) | 
								 ( ADD_SPI_CS0 ) | LASTXFER;	
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
	
//MSW 14 bits
	Data_To_Send = MSBFQ_0_BYTEWMASK |
								 (( fqBuffer.FQbuf >> 22 ) & 0x3F ) | 
								 ( ADD_SPI_CS0 );	
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );

	Data_To_Send = (( fqBuffer.FQbuf >> 14 ) & 0xFF ) | 
								 ( ADD_SPI_CS0 ) | LASTXFER;	
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
	while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
	
};

//++++++++++++++++++++++++++++++++++++++++++++

void RestoreAD9834fq ( void )
{
	fqBuffer.FQbuf = param.AD9834Afrequency;
	SendDivisor2DDS ( AT91C_BASE_SPI0 );
	UpdateDisplay();
};

//++++++++++++++++++++++++++++++++++++++++++++

void UpdateDisplay ( void )
{
	BYTE				idx;
	BYTE				jdx;
	BYTE				offset = 0;
	int					FQ4RIT = 0;
	int					FQ2displayUpper;
	int					FQ2displayLower;
	char				fq;
	WORD				mode = param.Modes & BFQmode;

	
	//Clean display buffer for first entry
	for ( idx = 0; idx <  13; idx++ )
		DisplayBuffer[ idx ] = ' ';
	for ( idx = CHAR_ON_ONE_LINE; idx < (DISP_POS_FILTER); idx++ )
		DisplayBuffer[ idx ] = ' ';
	if ( mode == AFQmode )
	{
		FQ2displayUpper = param.AD9834Afrequency;
		FQ2displayLower = param.AD9834Bfrequency;
		DisplayBuffer[ 0 ] = 'A';
		DisplayBuffer[ CHAR_ON_ONE_LINE ] = 'B';
	}
	else if ( mode == BFQmode )	
	{
		FQ2displayUpper = param.AD9834Bfrequency;
		FQ2displayLower = param.AD9834Afrequency;
		DisplayBuffer[ 0 ] = 'B';
		DisplayBuffer[ CHAR_ON_ONE_LINE ] = 'A';
	};
	if ( bSPLITFQmode )	
	{
		DisplayBuffer[ CHAR_ON_ONE_LINE ] = 'S';
	};
	if ( Band == Transverter )
		DisplayBuffer[ CHAR_ON_ONE_LINE ] = ' ';
	if  (( *AorBmode & RITmode ) == RITmode ) 
	{
		if ( mode == AFQmode )
		{
			FQ2displayUpper = param.AD9834Afrequency;
			DisplayBuffer[ 0 ] = 'A';
		}
		else if ( mode == BFQmode )
		{
//			FQ2displayLower = param.AD9834Bfrequency;
			FQ2displayUpper = param.AD9834Bfrequency;
			DisplayBuffer[ 0 ] = 'B';
		};
		DisplayBuffer[ CHAR_ON_ONE_LINE ] = 'R';
	};
	
	// For line 1 on display
	NumberOfDigits = ulong2string ( (char*)NumberBuffer, FQ2displayUpper );
//	if ( Band == Transverter )
	{
//	idx = 2;
//	offset = FQ_LASTPOSITION - NumberOfDigits + 2;
	}
//	else
	{
		idx = 0;
		offset = FQ_LASTPOSITION - NumberOfDigits;
	};
	for ( ; idx < ( NumberOfDigits - 1 ); idx++ ) //to - 1 20071102
	{
//		if (( offset == 3 ) || ( offset == 7 ))
		if ( offset == 7 )
		{
			DisplayBuffer [ offset++ ] = '.';
		};
		DisplayBuffer [ offset++ ] = NumberBuffer [ idx ];
	};	
	
	if ( Band == Transverter )
	{
/*		if ( mode == BFQmode )
		{
			DisplayBuffer[ 1 ] = 'B';
		}
		else
		{
			DisplayBuffer[ 1 ] = 'A';
		};
*/
    memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cTransverter [0],
        strlen ( cTransverter ));
		
		if (( param.Modes & RITmode ) == NonRITmode )
			return;
	};
		
	// For line 2 on display
	if (( *AorBmode & RITmode ) == RITmode )
	{
		if ( mode == AFQmode )
		{
		  FQ4RIT = FQ2displayUpper - AD9834RIT_Afrequency;
//20090920v
			if ( abs ( FQ4RIT ) >= MAX_RIT_DIFFERENCE )
			{
				if ( FQ4RIT > 0 )
				{
					FQ4RIT = MAX_RIT_DIFFERENCE - 50;
					AD9834RIT_Afrequency = FQ2displayUpper - MAX_RIT_DIFFERENCE;
				}
				else
				{
					FQ4RIT = - ( MAX_RIT_DIFFERENCE - 50 );
					AD9834RIT_Afrequency = FQ2displayUpper + MAX_RIT_DIFFERENCE;
				};
			};
		}
		else if ( mode == BFQmode )
		{
			FQ4RIT = FQ2displayUpper - AD9834RIT_Bfrequency;
			if ( abs ( FQ4RIT ) >= MAX_RIT_DIFFERENCE )
			{
				if ( FQ4RIT > 0 )
				{
					FQ4RIT = ( MAX_RIT_DIFFERENCE - 50 );
					AD9834RIT_Bfrequency = FQ2displayUpper - MAX_RIT_DIFFERENCE;
				}
				else
				{
					FQ4RIT = - ( MAX_RIT_DIFFERENCE - 50 );
					AD9834RIT_Bfrequency = FQ2displayUpper + MAX_RIT_DIFFERENCE;
				};
			};
		};
//20090920^
		NumberOfDigits = int2stringWsign ( (char*)NumberBuffer, FQ4RIT );
		offset = FQ_LASTPOSITION - NumberOfDigits + 
											CHAR_ON_ONE_LINE - 2;
		NumberOfDigits++;
	}
	else
	{
		NumberOfDigits = ulong2string ( (char*)NumberBuffer, FQ2displayLower );
		offset = FQ_LASTPOSITION - NumberOfDigits + CHAR_ON_ONE_LINE;
	};
	jdx = offset;
  for ( idx = 0; idx < (NumberOfDigits - 1 ); idx++ ) //to - 1 20071102
	{
		if (( *AorBmode & RITmode ) == NonRITmode )
		{
//			if (( jdx == 23 ) || ( jdx == 27 ))
			if ( jdx == (CHAR_ON_ONE_LINE + 7) )
			{
				DisplayBuffer [ jdx++ ] = '.';
			};
		};
		DisplayBuffer [ jdx++ ] = NumberBuffer [ idx ];
	};
	
	if ( FQ4RIT != 0 ) //Set sign if RIT difference 
	{
		if ( FQ4RIT > 0 )
			DisplayBuffer [ offset - 1 ] = '-';
		else
			DisplayBuffer [ offset - 1 ] = '+';
	};
	if (( *AorBmode & RITmode ) == RITmode )
	{
		DisplayBuffer [ jdx++ ] = 'H';
		DisplayBuffer [ jdx 	] = 'z';
	};

};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE readFromDDS ( BYTE register_in_DDS )
{
	UINT						SPI_msg;
	
	SPI_msg = AT91F_SPI_Read_Data ( AT91C_BASE_SPI0 );
	SPI_msg &= 0x0FF;
	return SPI_msg;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void 	init_DDS_chip ( const AT91PS_SPI pSpiIn )
{
	unsigned long Data_To_Send;
//	pSpi = pSpiIn;
	
	EnableDDS;
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
//	AT91F_PIO_SetOutput ( AT91C_BASE_PIOA, aDDS_CS );
// Cfg SPI MR
		// SPI is in master mode + CS not variable + CS directly connected + 
		// no Fault mode + min DLYBCS
		pSpiIn -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_SPI_CS0 );
		// For device 1 ( AD9834 ) set speed to MCKL/16 and 8 bits, 
		// CS active after transfer, mode 2, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 8 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
                    //20081008( 16 << 8 ) | 
										AT91C_SPI_BITS_8 | 
										AT91C_SPI_NCPHA | 	//Mode 2 
										AT91C_SPI_CPOL | 		//Mode 2 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR0 );
										
	Data_To_Send = MSBCONTROLBYTE_RESET | ( ADD_SPI_CS0 ); //  bit reset set
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
  //20081008while (spi_Tdre_Flag == 0);


	Data_To_Send = LSBCONTROLBYTE | ( ADD_SPI_CS0 ) | LASTXFER;										
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
  //20081008while (spi_Tdre_Flag == 0);
	
	Data_To_Send = MSBCONTROLBYTE | ( ADD_SPI_CS0 ); // reset not set
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
  //20081008while (spi_Tdre_Flag == 0);

	Data_To_Send = LSBCONTROLBYTE | ( ADD_SPI_CS0 ) | LASTXFER;										
	AT91F_SPI_Send_Data ( pSpiIn, Data_To_Send );
	AT91F_SPI_Read_Data (pSpiIn);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
  //20081008while (spi_Tdre_Flag == 0);
	
//	CalculateFrequencyDivisor ( param.AD9834Afrequency ); Done in Main.c!
	
	SendDivisor2DDS ( pSpiIn );
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void	CalculateFrequencyDivisor ( UINT frequency )
{
	volatile UINT	divisor = 0;
	volatile UINT	fq = frequency;
 						BYTE			Procedure;
 						
/*	
	switch ( Band )
	{
		case Transverter:  	Procedure = ADDING;		break;
		case B160:  				Procedure = ADDING; 	break;
		case B80:  					Procedure = ADDING; 	break;
		case B40:  					Procedure = ADDING; 	break;
		case B30:  					Procedure = ADDING; 	break;
		case B20: 					Procedure = ADDING; 	break;
		case B17:  					Procedure = ADDING; 	break;
		default:						Procedure = 0;				break;
	};
	if ( Procedure == ADDING )
	{

		fq += IFFREQUENCY;
	}
	else if ( Procedure == SUBTRACTING )
	{
		fq -= IFFREQUENCY;
	};
*/	
//	if ( param.Modes == BFQmode )
	if (( param.Modes & BFQmode) == BFQmode )	//20100117 Tobbe
	{
		if (( param.ModesB &  LSBmode ) == LSBmode )
		{

			fq += LSB_IFFREQUENCY;
		}
		if (( param.ModesB &  LSBmode ) == USBmode )
		{
			fq += USB_IFFREQUENCY;
		};

		if (( param.ModesB &  LSBmode ) == CWmode )
		{
			if ( mainmode == TXmode )
			{
				fq += LSB_IFFREQUENCY;
			}
			else
			{
				fq += CW_IFFREQUENCY;
			};
		};
	}
	else
	{
		if (( param.Modes &  LSBmode ) == LSBmode )
		{

			fq += LSB_IFFREQUENCY;
		}
		if (( param.Modes &  LSBmode ) == USBmode )
		{
			fq += USB_IFFREQUENCY;
		};

		if (( param.Modes &  LSBmode ) == CWmode )
		{
			if ( mainmode == TXmode )
			{
				fq += LSB_IFFREQUENCY;
			}
			else
			{
				fq += CW_IFFREQUENCY;
			};
		};
	};
	
	while ( fq > 9999999 )
	{
		divisor += RAM.f10MHz;
		fq -= 10000000;
	};
	while ( fq > 999999 )
	{
		divisor += RAM.f1MHz;
		fq -= 1000000;
	};
	while ( fq > 99999 )
	{
		divisor += RAM.f100kHz;
		fq -= 100000;
	};
	while ( fq > 9999 )
	{
		divisor += RAM.f10kHz;
		fq -= 10000;
	};
	while ( fq > 999 )
	{
		divisor += RAM.f1kHz;
		fq -= 1000;
	};
	while ( fq > 99 )
	{
		divisor += RAM.f100Hz;
		fq -= 100;
	};
	while ( fq > 9 )
	{
		divisor += RAM.f10Hz;
		fq -= 10;
	};
	while ( fq > 0 )
	{
		divisor += RAM.f1Hz;
		fq -= 1;
	};
	fqBuffer.FQbuf = divisor;
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void	CalculateDDSfractions ( UINT AD9834RefFrequency )
{
  uint64_t		help2to28		= 0x5F5E1000000000; // <= 0xA0000000 * 10000000
	UINT				helpRefFq 	= AD9834RefFrequency;
	uint64_t		helpFraction;
	
		helpFraction = help2to28;
		helpFraction /=  helpRefFq;
		RAM.f10MHz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f10MHz++;

		helpFraction /=  10;
		RAM.f1MHz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f1MHz++;

		helpFraction /=  10;
		RAM.f100kHz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f100kHz++;
			
		helpFraction /=  10;
		RAM.f10kHz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f10kHz++;

		helpFraction /=  10;
		RAM.f1kHz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f1kHz++;

		helpFraction /=  10;
		RAM.f100Hz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f100Hz++;

		helpFraction /=  10;
		RAM.f10Hz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f10Hz++;

		helpFraction /=  10;
		RAM.f1Hz 	= helpFraction / 10;
		if (( helpFraction % 10 ) > 4 )
			RAM.f1Hz++;


	send2EEPROM ( AddSavedFQvalues, (BYTE*)&RAM.f1Hz, sizeof ( struct FQvalues ));

};

//+++++++++++++++++++++++++++++++++++++++++++++++++

