//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt

/*----------------------------------------------------------------------------
* File Name           : display.c
// Object              : QROlleSAM7S Board Features Definition File.
//
// Creation            : NSn   20080111
*----------------------------------------------------------------------------
*/

// Include the board file description
#include "Board.h"
#include "menu.h"
#include "eepromhandler.h"
#include "display.h"

//Size in char's of display
#define CHAR_ON_DISPLAY       40
#define SHORT_DELAY           4

void SendCharacterPatternToDisplay ( BYTE 	data );


//++++++++++++++++++++++++++++++++++++++++++++
	volatile char					DisplayBuffer [ CHAR_ON_DISPLAY + 1 ];
	volatile char					LastDisplayBuffer [ CHAR_ON_DISPLAY + 1 ];
	volatile BYTE					lastDisplayPosition;
	volatile BYTE					DisplayCtrlPortValue = 0;
	volatile BYTE					EverySecondTime;
	volatile BYTE					temp;
	volatile BYTE					DisplayTiming;
	volatile UINT					TimeBetweenChars;
	volatile BYTE					ChangeSizePosition;
  volatile BYTE         NumberOfDigits; 
  volatile BYTE         EnableDelay;
  
//+++++++++++++++++++++++++++++++++++++++++++++++++
  
  void MakeShortDelay ( void )
  {  
    EnableDelay = SHORT_DELAY;
    while ( EnableDelay-- ); //Enable-pulse
  };
 
//++++++++++++++++++++++++++++++++++++++++++++++++++++
   
  void Make2string ( BYTE * data, BYTE value )
  {  
  BYTE  FirstDigit = 0;
  BYTE  SecondDigit = 0;
  BYTE  ThirdDigit = 0;
	
  for ( ; value > 99; value -= 100 )
   FirstDigit++;   
  for ( ; value > 9; value -= 10 )
   SecondDigit++;   
  for ( ; value > 0; value-- )
   ThirdDigit++;
	if ( FirstDigit > 0 )
		data [ 0 ] = 0x30 + FirstDigit;
	if (!(( FirstDigit == 0 ) && ( SecondDigit == 0 )))
		data [ 1 ] = 0x30 + SecondDigit;
	data [ 2 ] = 0x30 + ThirdDigit;
};


//+++++++++++++++++++++++++++++++++++++++++++++++++

void SendCmdToDisplay ( BYTE 	data, 
						BYTE 	delay )
{
	volatile	UINT		InternalDelay;
	
	SetWriteCommand;// Clear RW and Rsel  
	
  Write_Display_Data ( data );
  
  Toggle_Display_Latch;
  
	InternalDelay = DELAYBETWEENCOMMANDS;
	while ( InternalDelay-- ); //Distance between commands
	
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void SendCharacterPatternToDisplay ( BYTE 	data )
{
	volatile UINT	InternalDelay ;

	SetWriteCharacters;// Clear RW set Rsel  
   
  Write_Display_Data ( data );

  Toggle_Display_Latch;
	
	InternalDelay = DELAYBETWEENCOMMANDS;
	while ( InternalDelay-- ); //Distance between characters
	
//	TimeBetweenChars = EXTENDEDEXETIME;
//OneMore2:		
//	if ( TimeBetweenChars != 0 ) 
//		goto OneMore2;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void SendToDisplay ( 	char *	data, 
											BYTE 		position, 
											BYTE 		delay, 
											BYTE 		count )
{
	volatile BYTE			CountDown = count;	
	volatile BYTE			DispPos = position;	
	volatile BYTE			idx;	
	volatile BYTE			TXbuf;
	volatile UINT	InternalDelay;
	
	for ( idx = 0; idx < count; idx++ )
	{
		switch ( data [idx ] )
		{
			case '': data [idx ] = 0x61; break;
			case '': data [idx ] = 0xE1; break;
			case '': data [idx ] = 0xEF; break;
			case '': data [idx ] = 0x41; break;
			case '': data [idx ] = 0xE1; break;
			case '': data [idx ] = 0xEF; break;
			case '{': data [idx ] = 0x61; break;
			case '|': data [idx ] = 0xE1; break;
			case '}': data [idx ] = 0xEF; break;
			case '[': data [idx ] = 0x41; break;
			case 0x5C: data [idx ] = 0xE1; break;
			case ']': data [idx ] = 0xEF; break;
			default:;
		};
	};

OneMoreChar:
	if ( DispPos  == CHAR_ON_ONE_LINE )
		lastDisplayPosition = 0x40;
	if (( lastDisplayPosition + 1 ) == DispPos )
	{
		goto SkipPosition;
	};
	if ( DispPos != NO_POSITION )
	{
    if ( DispPos < CHAR_ON_ONE_LINE )
      TXbuf = ( DispPos + SETDDRAMADDRESS );
    else
      TXbuf = (( 0x40 - CHAR_ON_ONE_LINE + DispPos ) | SETDDRAMADDRESS ); 
    
    SetWriteCommand;// Clear RW and Rsel  
    
    Write_Display_Data ( TXbuf );
    
    Toggle_Display_Latch;
    
	
		InternalDelay = DELAYBETWEENDATA;
		while ( InternalDelay-- ); //Distance between characters
	};
	if ( count == 0 )
	{ 
		lastDisplayPosition = DispPos;
		return;
	};
SkipPosition:
  TXbuf = ( *data++ );

   SetWriteCharacters;// Clear RW set Rsel  
    
   Write_Display_Data ( TXbuf );

   Toggle_Display_Latch;
   
		
	if ( DispPos != NO_POSITION )
		lastDisplayPosition = DispPos++;

	InternalDelay = DELAYBETWEENDATA;
	while ( InternalDelay-- ); //Distance between characters
	
	if ( count == 0 )
	{ 
		lastDisplayPosition = DispPos;
		return;
	};
	if ( --CountDown ) 
		goto OneMoreChar;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE int2stringWsign ( char * OutString, int InValue  )
{
	UINT		UsedValue = abs ( InValue );
	
	return ulong2string ( OutString, UsedValue );
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

BYTE ulong2string ( char * OutString, unsigned int InValue  )
{
	unsigned int 			value = InValue;
	BYTE					digit;
	BYTE					digitindex = 0;
	
	digit = 0;
	while ( value > 999999999 )
	{
		value -= 1000000000;
		digit++;
	};
	if ( digit )
		OutString [ digitindex++ ] = digit + 0x30;
		
	digit = 0;
	while ( value > 99999999 )
	{
		value -= 100000000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 9999999 )
	{
		value -= 10000000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 999999 )
	{
		value -= 1000000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 99999 )
	{
		value -= 100000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 9999 )
	{
		value -= 10000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 999 )
	{
		value -= 1000;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 99 )
	{
		value -= 100;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value > 9 )
	{
		value -= 10;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	digit = 0;
	while ( value )
	{
		value -= 1;
		digit++;
	};
	if (( digitindex ) || ( digit ))
	{
		OutString [ digitindex++ ] = digit + 0x30;
	};
	return digitindex;
};


//+++++++++++++++++++++++++++++++++++++++++++++++++

void	SetCursorPosition ( BYTE Position )
{
	SendToDisplay ( (char*)&DisplayBuffer [ Position ], 
									Position, 
									NORMALEXETIME, 
									0 );
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void Write2display ( void )
{
	int				PosCursorPointer;
	BYTE			idx, changeflag = 0;
	int				CompSizePosition = ChangeSizePosition;
	
	for ( idx = 0; idx < CHAR_ON_DISPLAY; idx++ )
	{
		if ( DisplayBuffer [ idx ] != LastDisplayBuffer [ idx ] )
		{
			LastDisplayBuffer [ idx ] = DisplayBuffer [ idx ];
			SendToDisplay ( (char*)&DisplayBuffer [ idx ], 
											idx, 
											NORMALEXETIME, 
											1 );
			changeflag = 1;
		};
	};
	if ( mainmode == RXmode )
	{
		if ( EncoderSwitch_active )
			CursorPosition = POS_10kHz;
		else if ( param.FQstepLen == SMALL_FQ_STEPS )
			CursorPosition = POS_50Hz;
		else
			CursorPosition = POS_500Hz;
		if ( CheckButton ( BUTTONRIT, NOREPEAT )) //We use lower line FQ!
			CursorPosition + CHAR_ON_ONE_LINE;
	};
	if ( changeflag == 1 )  // Set cursor only if display is upated
		SetCursorPosition ( CursorPosition );
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

