//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
//
//  This code is distributed under the GNU Public License
//  which can be found at http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------
// File Name           : menu.c
// Object              : QROlleSAM7X Board Features Definition File.
//
// Creation            : NSn   20080111
//----------------------------------------------------------------------------
// Include the board file description
#include <string.h> 
#include <inttypes.h>

#include "Board.h"

#include "commands.h"
#include "display.h"
#include "buttons.h"
#include "TWI_ctrl.h"
#include "eepromhandler.h"
#include "HandleDDS.h"
#include "SPI_ctrl.h"
#include "Smeter.h"
#include "menu.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++

const char cLogTable [] = { 0, 6, 12, 20, 26, 34, 42, 50, 58, 66, 74, 84, 
														94, 104, 116, 126, 138, 152, 166, 180, 196, 214, 232, 255, 
														255, 255, 255, 255, 255, 255, 255 };
const char cPowerLogTable [] = { 37, 41, 45, 48, 51, 53, 55, 56, 57, 58, 59, 60, 
														61, 62, 63, 64, 65, 67, 69, 71, 73, 75, 77, 80, 
														84, 89, 97, 105, 120, 145, 183, 255, 255 };

//													 		 		 0123456789012345678901234567890123456789
const char cMenu [] 							=	"Menu: Use ^,v and > ";
const char cSetBaudrate [] 			=	"Baudrate";
const char cBaudrateNow [] 				=	"Baudrate now ";
const char cBugConnect		[]			=	"Bug polarity";
const char cBugConnectMode[]				=	"Bug polarity now ";
const char cSetButtonSound[]			=	"Button Sound ";
const char cButtonSound	 []				=	"Sound in ms";
const char cBreakInDelay [] 			=	"CW Breakin Delay";
const char cDelayNow [] 						=	"Delay now ";
const char cCWtraining []				=	"CW training ?";
const char cCWtrainingOn []				=	"is ON";
const char cCWtrainingOff []	 			=	"is OFF";
const char cSetDdsRefFrequency []=	"DDS Reference Freqv.";
const char cSetDisplayMode []		=	"Display mode";
const char cDisplayOff []					=	"Display low";
const char cDisplayOn []						=	"Display on";
const char cDisplayAuto []					=	"Display Auto";
const char cDVMRefStart	 []			=	"DVM value adjust?";
const char cDVMRef				 []				=	"Value now";
const char cIambic []						=	"Keying mode";
const char cIambicOff []						=	"Normal Bug";
const char cIambicA []							=	"Iambic mode A";
const char cIambicB []							=	"Iambic mode B";
const char cStraightKey []					=	"Straight Key";
const char cAccuKey []						=	"Accu Key";
const char cSoftware		 [] 			= "Program compiled";
const char cInit1 []							=	"Reset memory to QR- ";
const char cInit2 []								=	"Olle factory default";
const char cInitSure			 []				=	"Really really sure?";
const char cInitEnd1			 []				=	"Restart QROlle now ";
const char cSetSMeterCal [] 			=	"SMeter Calibration:";
const char cShowSMeterValue [] 		=	"SMeter shows";
const char cSMeterCalNow [] 				=	"S9  is set to";
const char cSaveWith [] 						=	"Save: Pressing knob!";
const char cSkipSaveWith [] 				=	"Skip Save: Hit MENU ";
const char cSMeterTyp []					=	"Smeter mode:";
const char cSMeterTypStart []			= "SMeter display: ";
const char cSMeterGraph []					=	"Graphics mode";
const char cSMeterNumeric []				=	"Numeric mode ";
const char cVoltages			 []			=	"Voltages: 9V =      ";
const char cVoltages2		 []				=	"5V=       3,3V=     ";
const char cUsb [] 							= "USB";
const char cLsb [] 							= "LSB";
const char cCW [] 								= "CW ";
//													 		 		 01234567890123456789
#define MENULEVELS      	20  
#define SUBLEVELS      		4 
void (* FunctionPointer [ MENULEVELS ][ SUBLEVELS ])();

//+++++++++++++++++++++++++++++++++++++++++++++++++

volatile BYTE					Idx, jdx;
volatile BYTE 				menuPosRow = 0;
volatile BYTE 				menuPosCol = 0;

volatile BYTE					KnobGoingActive = 0;
volatile BYTE 				ChangeDirection = 0;
volatile BYTE 				ValueChanged = 0;
volatile BYTE 				ControlChanged = 0;
volatile BYTE 				ADCchannel = 0;

volatile BYTE 				CursorJump = 0;
volatile BYTE 				handleMenuItem = 0;
volatile char					SaveBuffer [ CHAR_ON_DISPLAY ];
volatile BYTE					NumberBuffer [ CHAR_ON_DISPLAY ];
volatile BYTE 				CursorPosition = 0;
volatile UINT 				DivisorFrequency = 0;
volatile UINT 				DdsRefFrequencyAtStart = 0;
volatile UINT 				DDSfactors [ 9 ];
volatile BYTE 				DDSfactorStep = 0;
volatile BYTE 				Filter = 0;
volatile BYTE 				SavedPot1Volume = 0;
volatile BYTE 				subfunction = 0;
volatile WORD 				subfunctiontiming = 0;
volatile WORD 				ShowVoltage = 0;
volatile BYTE					UsedFQ = 0;
volatile BYTE					Band = 0;
volatile BYTE					Pot1Volume = 0;
volatile WORD					wUsedDelay = 0;
volatile BYTE					LastMenuRow = 1;
volatile BYTE					DecimalPosition = 0;
volatile BYTE					CWtraining = 0;
volatile BYTE					ShowBugSpeed = 0;
volatile BYTE 				ChangedData = 0;
volatile BYTE 				UpdateDDS = 0;
volatile WORD					wButtonSoundLen;
volatile BYTE 				bSPLITFQmode = 0;
volatile BYTE 				bSet2transmit = 0;
volatile BYTE 				bSMeterDisplay;
  
//+++++++++++++++++++++++++++++++++++++++++++++++++

void CompilerDate ( void  )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSoftware [0], strlen ( cSoftware ) );
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cVersion [0], strlen ( cVersion ) );
  DisplayBuffer [ CHAR_ON_ONE_LINE + 3 ] = GENERATION;
  DisplayBuffer [ CHAR_ON_ONE_LINE + 4 ] = COMBOGENERATION;
	memcpy ( (char*)&DisplayBuffer [ strlen ( cVersion ) + CHAR_ON_ONE_LINE + 1 ], 
												&cCompileDate [0], strlen ( cCompileDate ));
	CursorPosition = CHAR_ON_DISPLAY;
  if ( KnobGoingActive )
	{
		KnobGoingActive = false;
		AlreadyFound = true;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void MakeWord2string ( char * data, WORD value, BYTE position )
{  
  BYTE  FirstDigit = 0;
  BYTE  SecondDigit = 0;
  BYTE  ThirdDigit = 0;
  BYTE  FourthDigit = 0;
	
  for ( ; value > 999; value -= 1000 )
   FirstDigit++;   
  for ( ; value > 99; value -= 100 )
   SecondDigit++;   
  for ( ; value > 9; value -= 10 )
   ThirdDigit++;
  for ( ; value > 0; value-- )
   FourthDigit++;
	if ( FirstDigit > 0 )
		data [ position ] = 0x30 + FirstDigit;
	if (!(( FirstDigit == 0 ) && ( SecondDigit == 0 )))
		data [ position + 1 ] = 0x30 + SecondDigit;
	if (!(( FirstDigit == 0 ) && ( SecondDigit == 0 ) && ( ThirdDigit == 0 )))
		data [ position + 2 ] = 0x30 + ThirdDigit;
	data [ position + 3 ] = 0x30 + FourthDigit;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void MakeByte2string ( char * data, BYTE value, BYTE position )
{  
  BYTE  FirstDigit = 0;
  BYTE  SecondDigit = 0;
  BYTE  ThirdDigit = 0;
	
  for ( ; value > 99; value -= 100 )
   FirstDigit++;   
  for ( ; value > 9; value -= 10 )
   SecondDigit++;   
  for ( ; value > 0; value-- )
   ThirdDigit++;
	if ( FirstDigit > 0 )
		data [ position ] = 0x30 + FirstDigit;
	if (!(( FirstDigit == 0 ) && ( SecondDigit == 0 )))
		data [ position + 1 ] = 0x30 + SecondDigit;
	data [ position + 2 ] = 0x30 + ThirdDigit;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void Make2HEXstringByte ( char * data, BYTE value, BYTE position )
{  
  BYTE  FirstDigit = 0;
  BYTE  SecondDigit = 0;
	
  data [ position ] = 0x30;
  data [ position + 1 ] = 'x';

	for ( ; value > 15; value -= 16 )
   FirstDigit++;   
  for ( ; value > 0; value -= 1 )
   SecondDigit++;   

	FirstDigit += 0x30;
	if ( FirstDigit > 0x39 )
		FirstDigit += ( 0x10 - 9 );
	data [ position + 2 ] = FirstDigit;
	SecondDigit += 0x30;
	if ( SecondDigit > 0x39 )
		SecondDigit += ( 0x10 - 9 );
	data [ position + 3 ] = SecondDigit;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void Make2HEXstringWORD ( char * data, WORD value, BYTE position )
{  
  BYTE  FirstDigit = 0;
  BYTE  SecondDigit = 0;
  BYTE  ThirdDigit = 0;
  BYTE  FourthDigit = 0;
	
  data [ position ] = 0x30;
  data [ position + 1 ] = 'x';

	for ( ; value > 4095; value -= 4096 )
   FirstDigit++;   
  for ( ; value > 255; value -= 256 )
   SecondDigit++;   
	for ( ; value > 15; value -= 16 )
   ThirdDigit++;   
  for ( ; value > 0; value -= 1 )
   FourthDigit++;   

	if ( FirstDigit == 0 )
	{
		data [ position + 2 ] = ' ';
	}
	else
	{
		FirstDigit += 0x30;
		if ( FirstDigit > 0x39 )
			FirstDigit += ( 0x10 - 9 );
		data [ position + 2 ] = FirstDigit;
	};
	if (( FirstDigit == 0 ) && ( SecondDigit == 0 ))
	{
		data [ position + 3 ] = ' ';
	}
	else
	{
		SecondDigit += 0x30;
		if ( SecondDigit > 0x39 )
			SecondDigit += ( 0x10 - 9 );
		data [ position + 3 ] = SecondDigit;
	};
	if (( FirstDigit == 0 ) && ( SecondDigit == 0 ))
	{
		data [ position + 4 ] = ' ';
	}
	else
	{
		ThirdDigit += 0x30;
		if ( ThirdDigit > 0x39 )
			ThirdDigit += ( 0x10 - 9 );
		data [ position + 4 ] = ThirdDigit;
	};
	FourthDigit += 0x30;
	if ( FourthDigit > 0x39 )
		FourthDigit += ( 0x10 - 9 );
	data [ position + 5 ] = FourthDigit;
};

//++++++++++++++++++++++++++++++++++++++++++++

BYTE	CheckEncoderWORD ( WORD volatile * Value2adjust, WORD MaxValue, WORD steps )
{
	BYTE 				Change = 0;
		
	if (( ControlChanged == SOUND_UP ) &&
		( *Value2adjust <= ( MaxValue - steps ) ))
	{
		(*Value2adjust)+= steps;
		Change = 0xFF;
	};
	if (( ControlChanged == SOUND_DOWN ) &&
		( *Value2adjust > 0 ))
	{
		(*Value2adjust)-= steps;
		Change = 0xFF;
	};
	ControlChanged = 0;
	return Change;
};

//++++++++++++++++++++++++++++++++++++++++++++

BYTE	CheckEncoderBYTE ( BYTE * Value2adjust, BYTE MaxValue, BYTE steps )
{
	BYTE 				Change = 0;
	
	if (( ControlChanged == SOUND_UP ) &&
		( *Value2adjust <= ( MaxValue - steps )))
	{
		(*Value2adjust)+= steps;
		Change = CONTROLCHANGED;
	};
	if (( ControlChanged == SOUND_DOWN ) &&
		( *Value2adjust >= steps ))
	{
		(*Value2adjust)-= steps;
		Change = CONTROLCHANGED;
	};
	if ((*Value2adjust) > MaxValue )
		*Value2adjust = MaxValue;
	if ((*Value2adjust) < steps )
		*Value2adjust = 0;
	ControlChanged = 0;
	return Change;
};

//++++++++++++++++++++++++++++++++++++++++++++

BYTE	CheckEncoderLONGWORD ( UINT * Value2adjust, UINT increment )
{
	BYTE 				Change = 0;
	
	if ( ControlChanged == SOUND_UP )
	{
		(*Value2adjust) = (*Value2adjust) + increment;
		Change = CONTROLCHANGED;
	};
	if ( ControlChanged == SOUND_DOWN )
	{
		(*Value2adjust) = (*Value2adjust) - increment;
		Change = CONTROLCHANGED;
	};
	ControlChanged = 0;
	return Change;
	
};

//++++++++++++++++++++++++++++++++++++++++++++

void DisplayValues ( void )
{
	WORD			Tempvalue;
	int				idx;
	
UINT			SMeterValue;
	

	if ( mainmode != MENUMODE )
	{
		UpdateDisplay (); // Frequency
	
		if ((( *AorBmode & NARROWmode ) == 0 ) || ( mainmode == TXmode ))
		{
			DisplayBuffer [ DISP_POS_FILTER ] 		= 'W';
			SetBitIOBW;
		}
		else
		{
			DisplayBuffer [ DISP_POS_FILTER ] 		= 'N';
			ResetBitIOBW;
		};
		if (( *AorBmode & PreAmp ) == PreAmp )
		{
			DisplayBuffer [ DISP_POS_PREAMP ] 		= 'P';
			SetBitIOHF;
		}
		else
		{
			DisplayBuffer [ DISP_POS_PREAMP ] 		= ' ';
			ResetBitIOHF;
		};
		if (( *AorBmode & Compress ) == Compress )
		{
			DisplayBuffer [ DISP_POS_COMPR ] 		= 'C';
			SetBitIOKO;
		}
		else
		{
			DisplayBuffer [ DISP_POS_COMPR ] 		= ' ';
			ResetBitIOKO;
		};
		if (( *AorBmode & LSBmode ) == USBmode )
		{
			DisplayBuffer [ DISP_POS_MODE ] 		= cUsb [ 0 ];
			DisplayBuffer [ DISP_POS_MODE + 1 ] = cUsb [ 1 ];
			DisplayBuffer [ DISP_POS_MODE + 2 ] = cUsb [ 2 ];
		}
		else if (( *AorBmode & LSBmode ) == LSBmode )
		{
			DisplayBuffer [ DISP_POS_MODE ] 		= cLsb [ 0 ];
			DisplayBuffer [ DISP_POS_MODE + 1 ] = cLsb [ 1 ];
			DisplayBuffer [ DISP_POS_MODE + 2 ] = cLsb [ 2 ];
		}
		else 	
		{
			DisplayBuffer [ DISP_POS_MODE ] 		= cCW [ 0 ];
			DisplayBuffer [ DISP_POS_MODE + 1 ] = cCW [ 1 ];
			DisplayBuffer [ DISP_POS_MODE + 2 ] = cCW [ 2 ];
		};
		CursorPosition = CHAR_ON_DISPLAY;
	};
};



//++++++++++++++++++++++++++++++++++++++++++++

void SetupCWtraining ( void )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cCWtraining[0], strlen ( cCWtraining ) - 1 );
	
	if ( CheckButton ( BUTTONBup, NOREPEAT ))
	{
		ChangedData = CheckEncoderWORD ( &wBugSpeed, MAXBUGSPEED, 1 );
		if ( ChangedData > 0 )
		{
			if ( wBugSpeed < MINBUGSPEED )
				wBugSpeed = MINBUGSPEED;
			ShowBugSpeed = SHOWBUGSPEED;
			bBugSpeedChange = 1;	
			SetTimer4Bug ( wBugSpeed );
		};
	};
//20081207	->	
	if ( CWtraining == 1 )
	{
		if ( IsIOEX_reset )
		{
			SetBitIOEX;
			ResetIOMO;
		};
		SetTimer4Bug ( wBugSpeed );
		memcpy ( (char*)&DisplayBuffer[ CHAR_ON_ONE_LINE ], 
										&cCWtrainingOn[0], 
										strlen ( cCWtrainingOn ) );
	};
	if ( CWtraining == 0 )
	{
		ResetIOEX;
		if (( *AorBmode &  LSBmode ) != CWmode )
			SetIOMO;
		memcpy ( (char*)&DisplayBuffer[ CHAR_ON_ONE_LINE ], 
										&cCWtrainingOff[0], 
										strlen ( cCWtrainingOff ) );
	};
	if ( menuPosCol == 0 ) 
		return;
//20081207	<-	
	if ( CheckEncoderBYTE ( (BYTE*)&CWtraining, 1, 1 )) 
	{
	};
	if ( KnobGoingActive )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		menuPosCol = 0;
		if ( menuPosRow > 1 )
			menuPosRow--;
		ResetIOEX;
		if (( *AorBmode &  LSBmode ) != CWmode )
			SetIOMO;
	};
};

//++++++++++++++++++++++++++++++++++++++++++++

void Initialization ( void )
{
  if ( menuPosCol == 0 ) 
  {
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cInit1 [0], strlen ( cInit1 ));
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cInit2 [0], strlen ( cInit2 ));
	};
  if (( menuPosCol == 1 ) || ( menuPosCol == 2 ))
  {
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cInitSure [0], strlen ( cInitSure ));
		if ( KnobGoingActive )
		{
			KnobGoingActive = false;
			if ( menuPosCol < ( SUBLEVELS - 1 ))
			{
				menuPosCol++;
			};
		};
	};
  if ( menuPosCol == 3 ) 
  {
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		memcpy ( (char*)&DisplayBuffer [ 0 ], &cInitEnd1 [0], strlen ( cInitEnd1 ));
		wCheck4eeprom = 0x0;
		send2EEPROM ( AddwCheck4eeprom, (BYTE*)&wCheck4eeprom, 2 );
	};
};

//++++++++++++++++++++++++++++++++++++++++++++

void JumpBackAddress ( void )
{
	menuPosRow = 1;
	menuPosCol = 0;
	
};

//++++++++++++++++++++++++++++++++++++++++++++

void InitMenu ( void )
{
	BYTE	idx, resetValue = 0;

	SetPot1Volume();
	SetAD9834value();
	DisplayValues ();
	
//	memset ( &FunctionPointer, 0, sizeof ( FunctionPointer ));
	for ( Idx = 0; Idx < MENULEVELS; Idx++ )
		for ( jdx = 0; jdx < SUBLEVELS; jdx++ )
			FunctionPointer [ Idx ][ jdx ] = NULL;

	//20090921  ->  
	FunctionPointer [  0 ][ 0 ] = 0;//Avoiding problems above...
	FunctionPointer [  1 ][ 0 ] = SetBaudRate;
	FunctionPointer [  1 ][ 1 ] = SetBaudRate;
	FunctionPointer [  2 ][ 0 ] = SetBugConnect;
	FunctionPointer [  2 ][ 1 ] = SetBugConnect;
	FunctionPointer [  3 ][ 0 ] = SetButtonSound;
	FunctionPointer [  3 ][ 1 ] = SetButtonSound;
	FunctionPointer [  4 ][ 0 ] = BreakInDelay;
	FunctionPointer [  4 ][ 1 ] = BreakInDelay;
	FunctionPointer [  5 ][ 0 ] = SetupCWtraining;
	FunctionPointer [  5 ][ 1 ] = SetupCWtraining;
  FunctionPointer [  6 ][ 0 ] = SetDdsRefFrequency;
	FunctionPointer [  6 ][ 1 ] = SetDdsRefFrequency;
	FunctionPointer [  7 ][ 0 ] = SetDisplayMode;
	FunctionPointer [  7 ][ 1 ] = SetDisplayMode;
	FunctionPointer [  8 ][ 0 ] = SetupDVM;
	FunctionPointer [  8 ][ 1 ] = SetupDVM;
	FunctionPointer [  9 ][ 0 ] = SetIambicMode;
	FunctionPointer [  9 ][ 1 ] = SetIambicMode;
	FunctionPointer [ 10 ][ 0 ] = CompilerDate;
	FunctionPointer [ 11 ][ 0 ] = Initialization;
	FunctionPointer [ 11 ][ 1 ] = Initialization;
	FunctionPointer [ 11 ][ 2 ] = Initialization;
	FunctionPointer [ 11 ][ 3 ] = Initialization;
	FunctionPointer [ 12 ][ 0 ] = SetSMeterCal;
	FunctionPointer [ 12 ][ 1 ] = SetSMeterCal;
	FunctionPointer [ 12 ][ 2 ] = SetSMeterCal;
	FunctionPointer [ 13 ][ 0 ] = ShowSMeterCal;
	FunctionPointer [ 13 ][ 1 ] = ShowSMeterCal;
	FunctionPointer [ 13 ][ 2 ] = ShowSMeterCal;
	FunctionPointer [ 14 ][ 0 ] = SetupSMetertype;
	FunctionPointer [ 14 ][ 1 ] = SetupSMetertype;
	FunctionPointer [ 15 ][ 0 ] = Voltages;
	//20090921  <-  
	FunctionPointer [  16 ][ 0 ] = JumpBackAddress;
#define LASTUSEDROWVALUE 16
	// If bigger array: extend array declaration!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void BreakInDelay ( void )
{
	WORD	value;
	
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cBreakInDelay [0], strlen ( cBreakInDelay ));
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cDelayNow[0], 10 );
	
	MakeWord2string ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE + 10 ], wUsedDelay, 4 );
	if ( menuPosCol == 0 ) 
		return;
	if ( CheckEncoderWORD ( (WORD*)&wUsedDelay, MAXDELAY, 10 )) 
	{
		if ( wUsedDelay < MIN_USED_DELAY )
			wUsedDelay = MIN_USED_DELAY;
	};
	if ( KnobGoingActive )
	{
		send2EEPROM ( AddwUsedDelay, (BYTE*)&wUsedDelay, 2 );
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void	SetStepSize ( BYTE direction )
{
	if ( direction == MOVEUP )
	{
		if ( ChangeSizePosition < NOMORECHANGESTEP )
			ChangeSizePosition++;
	}
	else
	{
		if ( ChangeSizePosition > 0 )
			ChangeSizePosition--;
	};
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SetPot1Volume ( void )
{
	volatile 	BYTE			ChangedData = 0;
						UINT			Data_To_Send;
						BYTE			LogValue, TableOffset;
	EnablePot;
	POT1dataDirty = 1;
  TableOffset = Pot1Volume;
  TableOffset /= VOLUMEPOT_STEPSIZE;
	LogValue = cLogTable [ TableOffset ];
	
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
	// Cfg SPI MR
	// SPI is in master mode + CS not variable + CS directly connected + 
	// no Fault mode + min DLYBCS
	AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_POT );
	// For device 0 ( Combodevices ) set speed to MCKL/16 and 10 bits, 
	// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 16 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_10 | 
										AT91C_SPI_NCPHA | 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR0 );
	Data_To_Send = ( LogValue + 0x100 ) | ( ADD_POT ) | LASTXFER;
//	Data_To_Send = LogValue | ( ADD_POT ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SetPowerLevel ( BYTE PowerSetting )
{
	volatile 	BYTE			ChangedData = 0;
						UINT			Data_To_Send;
	EnablePot;
 	
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
	// Cfg SPI MR
	// SPI is in master mode + CS not variable + CS directly connected + 
	// no Fault mode + min DLYBCS
	AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_POT );
	// For device 0 ( Combodevices ) set speed to MCKL/16 and 10 bits, 
	// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 16 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_10 | 
										AT91C_SPI_NCPHA | 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR0 );

//	Data_To_Send = ( param.bPowerSetting + 0x100 ) | ( ADD_POT ) | LASTXFER;
	Data_To_Send = ( cPowerLogTable [ PowerSetting ]) | ( ADD_POT ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
	
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SetTuningPower ( BYTE TuningPower )
{
	volatile 	BYTE			ChangedData = 0;
						UINT			Data_To_Send;
	EnablePot;
 	
	WaitUntilReady4TX ( AT91C_BASE_SPI0 );//20090414
	// Cfg SPI MR
	// SPI is in master mode + CS not variable + CS directly connected + 
	// no Fault mode + min DLYBCS
	AT91C_BASE_SPI0 -> SPI_MR = (AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | ADD_POT );
	// For device 0 ( Combodevices ) set speed to MCKL/16 and 10 bits, 
	// CS active after transfer, mode 0, 1 clk delay b4 CS up
	AT91F_Set_SPI_CSR ( AT91C_BASE_SPI0, 
//20090315										( 16 << 8 ) | 
										( SPI_clock_size_Combo << 8 ) | 
										AT91C_SPI_BITS_10 | 
										AT91C_SPI_NCPHA | 
										AT91C_SPI_CSAAT | 
										( 1 << 24 ), 
										CSR0 );

	Data_To_Send = ( TuningPower) | ( ADD_POT ) | LASTXFER;
	AT91F_SPI_Send_Data ( AT91C_BASE_SPI0, Data_To_Send );
	AT91F_SPI_Read_Data (AT91C_BASE_SPI0);
  while ( AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_TDRE == 0 );
	
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SetAD9834value ( void )
{
	volatile BYTE			ChangedData = 0;
	volatile BYTE			idx, count;
	volatile char			VFO = 'A';
	UINT							*	FQ2change;

	if ( mainmode == RXmode )
	{
		if (( param.Modes & BFQmode ) == AFQmode )
		{
			if (( *AorBmode & RITmode ) == RITmode )
			{
				FQ2change = (UINT*)&AD9834RIT_Afrequency;
				VFO = 'A';
			}
			else
			{
				if ( CheckButton ( BUTTONRIT, NOREPEAT ))
				{
					FQ2change = (UINT*)&param.AD9834Bfrequency;
					VFO = 'B';
				}
				else
				{
					FQ2change = (UINT*)&param.AD9834Afrequency;
					VFO = 'A';
				}
			};
		}
		else if (( param.Modes & BFQmode ) == BFQmode )	
		{
			if (( *AorBmode & RITmode ) == RITmode ) 
			{
				FQ2change = (UINT*)&AD9834RIT_Bfrequency;
				VFO = 'B';
			}
			else
			{
				if ( CheckButton ( BUTTONRIT, NOREPEAT ))
				{
					FQ2change = (UINT*)&param.AD9834Afrequency;
					VFO = 'A';
				}
				else
				{
					FQ2change = (UINT*)&param.AD9834Bfrequency;
					VFO = 'B';
				};
			};
		};

		if ( EncoderSwitch_active )
			ChangedData = CheckEncoderLONGWORD ( (UINT*)FQ2change, BIG_FQ_STEPS );
		else
		{
			ChangedData = CheckEncoderLONGWORD ( FQ2change, param.FQstepLen );
		};

	};

	if ( mainmode == TXmode )
	{
		if ( LastValidInport ) // If any button pressed: skip FQ change...
			return;

		if (( param.Modes & BFQmode ) == AFQmode )
		{
			if ( bSPLITFQmode )	
			{
				FQ2change = (UINT*)&param.AD9834Bfrequency;
			}
			else
			{
				FQ2change = (UINT*)&param.AD9834Afrequency;
			};
		}
		else if (( param.Modes & BFQmode ) == BFQmode )	
		{
			if ( bSPLITFQmode )	
			{
				FQ2change = (UINT*)&param.AD9834Afrequency;
			}
			else
			{
				FQ2change = (UINT*)&param.AD9834Bfrequency;
			};
		};
	};

//	if ((( ChangedData == CONTROLCHANGED ) || ( ModeChanged )) && ( mainmode == RXmode ))//20090920 
	if (( ChangedData == CONTROLCHANGED ) || ( ModeChanged ))//20100116 Tobbe

	{
//		if ( DisplayMode == DisplayAuto ) //then light display another 10 seconds
		if (( DisplayMode == DisplayAuto ) && ( ChangedData == CONTROLCHANGED)) //then light display another 10 seconds 20100116 Tobbe

		{
			if ( wDisplayBackLiteTime == 0 )
			{
				SwitchONbacklite;		
			};
			wDisplayBackLiteTime = TEN_SECONDS_DELAY;
		};
		Check4bandlimits ( FQ2change, Band );
		ModeChanged = 0;
		CalculateFrequencyDivisor ( *FQ2change );
		SendDivisor2DDS ( AT91C_BASE_SPI0 );
		if ( ChangedData == CONTROLCHANGED ) 
			AD9834dataDirty = 1;
		UpdateDisplay (); // Frequency
		ChangedData = 0;//20090331
		if ( AutoInformationFlag == 1 )
		{
			OutBuffer [ 0 ] = 'F';
			OutBuffer [ 1 ] = VFO;
			OutBuffer [ 2 ] = '0';
			OutBuffer [ 10 ] = ';';
			
			count = ulong2string ( (char*)&NumberBuffer, *FQ2change );
			for ( jdx = 0; 
						 jdx < count; 
						 jdx++ )
			{
				OutBuffer [ 2 + jdx + 8 - count ] = NumberBuffer [ jdx ];
			};
			for ( count = 0; count < ( 11 ); count++ )
				TransmitByte ( OutBuffer [ count ]);
		};
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void	Check4bandlimits ( UINT * FQ2change, BYTE band )
{
	UINT		LowerBandLimit;
	UINT		UpperBandLimit;
	
	switch ( band ) 
	{
		case B160:				LowerBandLimit = Lower160mLimit; 
											UpperBandLimit = Upper160mLimit; break;
		case B80:					LowerBandLimit = Lower80mLimit; 
											UpperBandLimit = Upper80mLimit; break;
		case B40:					LowerBandLimit = Lower40mLimit; 
											UpperBandLimit = Upper40mLimit; break;
		case B30:					LowerBandLimit = Lower30mLimit; 
											UpperBandLimit = Upper30mLimit; break;
		case B20:					LowerBandLimit = Lower20mLimit; 
											UpperBandLimit = Upper20mLimit; break;
		case B17:					LowerBandLimit = Lower17mLimit; 
											UpperBandLimit = Upper17mLimit; break;
		case Transverter:	LowerBandLimit = LowerTransverterLimit; 
											UpperBandLimit = UpperTransverterLimit; break;
	};
	if ( *FQ2change < LowerBandLimit )
	{
		memset ( (BYTE*)LastDisplayBuffer, ' ', CHAR_ON_DISPLAY );
		*FQ2change = LowerBandLimit;
	};
	if ( *FQ2change > UpperBandLimit )
	{
		memset ( (BYTE*)LastDisplayBuffer, ' ', CHAR_ON_DISPLAY );
		*FQ2change = UpperBandLimit;
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetBandData ( BYTE band )
{
	WORD		offsetIntoEEPROM;
	
	switch ( band ) 
	{
		case Transverter:	offsetIntoEEPROM = SavedParametersTrans; break;
		case B160:				offsetIntoEEPROM = SavedParametersB160;	break;
		case B80:					offsetIntoEEPROM = SavedParametersB80;	break;
		case B40:					offsetIntoEEPROM = SavedParametersB40;	break;
		case B30:					offsetIntoEEPROM = SavedParametersB30;	break;
		case B20:					offsetIntoEEPROM = SavedParametersB20;	break;
		case B17:					offsetIntoEEPROM = SavedParametersB17;	break;
	};
	readFromEEPROM ( offsetIntoEEPROM, (BYTE*)&param, sizeof ( struct Parameters ));
	if (( param.Modes &  BFQmode ) == BFQmode )
	{
		AorBmode = &param.ModesB;
	}
	else
	{
		AorBmode = &param.Modes;
	};
	if (( *AorBmode &  LSBmode ) == LSBmode )
	{
		SetBitIOSB;
		SetBitIOMO;
	};
	if (( *AorBmode &  LSBmode ) == CWmode )
	{
		SetBitIOSB;
		ResetBitIOMO;
	};
	if (( *AorBmode &  LSBmode ) == USBmode )
	{
		ResetBitIOSB;
		SetBitIOMO;
	};
	SetAD9834value();
	DisplayValues (); // Frequency
	SetPowerLevel(param.bPowerSetting);
	if (param.bTuningSetting > MAXTUNINGSETTING)
	{
		param.bTuningSetting = MAXTUNINGSETTING;
	}
	if (param.bTuningSetting < 0)
	{
		param.bTuningSetting = 0;
	}
	ResetFQbandBits;
	Write_Combo_IO ( A_PORT, (BYTE*)&param.IOportA, 1 );
	Write_Combo_IO ( B_PORT, (BYTE*)&param.IOportB, 1 );
	switch ( band ) 
	{
		case Transverter:	SetIOTV;SetIO20; break;
		case B160:				SetIO160;	break;
		case B80:					SetIO80;	break;
		case B40:					SetIO40;	break;
		case B30:					SetIO30;	break;
		case B20:					SetIO20;	break;
		case B17:					SetIO17;	break;
	};
	
	send2EEPROM ( LastBand, (BYTE*)&Band, 1 );
};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SaveBandData ( BYTE band, BYTE OffsetInStruct, BYTE WriteSize )
{
	WORD		offsetIntoEEPROM;

	switch ( band ) 
	{
		case Transverter:			offsetIntoEEPROM = SavedParametersTrans;break;
		case B160:						offsetIntoEEPROM = SavedParametersB160;break;
		case B80:							offsetIntoEEPROM = SavedParametersB80;break;
		case B40:							offsetIntoEEPROM = SavedParametersB40;break;
		case B30:							offsetIntoEEPROM = SavedParametersB30;break;
		case B20:							offsetIntoEEPROM = SavedParametersB20;break;
		case B17:							offsetIntoEEPROM = SavedParametersB17;break;
	};
	send2EEPROM (( offsetIntoEEPROM + OffsetInStruct ), 
							((BYTE*)&param ) + OffsetInStruct, 
							WriteSize );
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void Menu ( void )
{

	BYTE				idx, bMenuSteps = 2;

	if ( mainmode == MENUMODE )
	{

					
		if (( DisplayMode == DisplayAuto ) ||
				( DisplayMode == DisplayOff )) //then light display another 10 seconds
		{
			if ( wDisplayBackLiteTime  < 4 )
			{
				SwitchONbacklite;		
			};
			wDisplayBackLiteTime = TEN_SECONDS_DELAY;
		};
	};
	// Methods for subfunctions with buttons still pressed
	if ( LastValidInport )  //i.e. A button pressed
	{
		if ( DisplayMode == DisplayAuto ) //then light display another 10 seconds
		{
			if ( wDisplayBackLiteTime < 4 )
			{
				SwitchONbacklite;		
			};
			wDisplayBackLiteTime = TEN_SECONDS_DELAY;
		};
		if (( subfunction == BUTTONRIT ) && 
			( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON ))
		{
			if  ( !UpdateDDS )
			{
				UpdateDDS = 1;
				ModeChanged = 1;
				SetAD9834value();
			};
		};
		if ( CheckButton ( BUTTONBdw, NOREPEAT ))
		{
			if (LowPowerSet)
			{
				ShowPower = SHOWPOWERDELAY;	
				ChangedData = CheckEncoderBYTE ( (BYTE*)&param.bTuningSetting, MAXTUNINGSETTING, 1 );
				if ( ChangedData > 0 )
				{
					PowerdataDirty = 1;
					SetTuningPower(param.bTuningSetting);
					subfunctiontiming = 0;
					subfunction = 0;
				};
				
			}
			else
			{
				ShowPower = SHOWPOWERDELAY;	
				ChangedData = CheckEncoderBYTE ( (BYTE*)&param.bPowerSetting, MAXPOWERSETTING, 1 );
				if ( ChangedData > 0 )
				{
					PowerdataDirty = 1;
					SetPowerLevel(param.bPowerSetting);
					subfunctiontiming = 0;
					subfunction = 0;
				};
			};
		};
		if ( CheckButton ( BUTTONStep, NOREPEAT ))
		{
			if ( KnobGoingActive )
			{
				if ( Pot1Volume == 0 )
					Pot1Volume = SavedPot1Volume;	
				else
				{
					SavedPot1Volume = Pot1Volume;	
					Pot1Volume = 0; // MUTE-function
				};
				KnobGoingActive = false;
//				AlreadyFound = true;
				ShowVolume = SHOWVOLUMEDELAY;
				SetPot1Volume();
				subfunctiontiming = 0;
				subfunction = 0;
			};
//			ChangedData = CheckEncoderBYTE ( (BYTE*)&Pot1Volume, MAXVOLUME, MAXVOLUME/32 );
			ShowVolume = SHOWVOLUMEDELAY;
			ChangedData = CheckEncoderBYTE ( (BYTE*)&Pot1Volume, MAXVOLUME, MAXVOLUME/24 );
			if ( ChangedData > 0 )
			{
				POT1dataDirty = 1;
				ShowVolume = SHOWVOLUMEDELAY;
				SetPot1Volume();
				subfunctiontiming = 0;
				subfunction = 0;
			};
		};
		if ( CheckButton ( BUTTONBup, NOREPEAT ))
		{
			ChangedData = CheckEncoderWORD ( &wBugSpeed, MAXBUGSPEED, 1 );
			if ( ChangedData > 0 )
			{
				if ( wBugSpeed < MINBUGSPEED )
					wBugSpeed = MINBUGSPEED;
				bBugSpeedChange = 1;
				ShowBugSpeed = SHOWBUGSPEED;	
//				if (( *AorBmode & LSBmode ) == CWmode )
				SetTimer4Bug ( wBugSpeed );
				subfunctiontiming = 0;
				subfunction = 0;
			};
		};
	};

	if ( !( LastValidInport ))  //i.e. No button pressed, or all released
	{
		if ( subfunction == BUTTONFlt )
		{
			if ( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON )
			{
				ShowVoltage = TEN_SECONDS_DELAY;
			}
			else
			{
				if (( *AorBmode & NARROWmode ) == 0 )
				{
					*AorBmode |= NARROWmode;
					ResetIOBW;
				}
				else
				{
					*AorBmode &= ~NARROWmode;
					SetIOBW;
				};
			};
			subfunctiontiming = 0;
			subfunction = 0;
			OutputdataDirty = 1;
		};
		if ( subfunction == BUTTONPre )
		{
			if ( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON )
			{
				if ( ( *AorBmode & Compress ) == Compress )
				{
					*AorBmode &= ~Compress;
					ResetIOKO;
				}
				else
				{
					*AorBmode |= Compress;
					SetIOKO;
				};
			}
			else
			{
				if (( *AorBmode & PreAmp ) == PreAmp )
				{
					*AorBmode &= ~PreAmp;
					ResetIOHF;
				}
				else
				{
					*AorBmode |= PreAmp;
					SetIOHF;
				};
			};
			ModeChanged = 1;  // Change 
			OutputdataDirty = 1; //Set/reset IO-bits
			subfunction = 0;
			subfunctiontiming = 0;
			goto SetDefault;
		};
		if ( subfunction == BUTTONMd )
		{
			if ( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON )
			{
				mainmode = MENUMODE;
				memset ( (char*)&DisplayBuffer[0], ' ', CHAR_ON_DISPLAY ); 
				memcpy ( (char*)&DisplayBuffer[20], &cMenu[0], strlen ( cMenu ));
				menuPosRow = LastMenuRow;
				menuPosCol = 0;
			}
			else
			{
				if (( *AorBmode &  LSBmode ) == LSBmode )
				{
					*AorBmode &= ~LSBmode;
					*AorBmode |= USBmode;
					ResetBitIOSB;
					SetIOMO;
				}
				else if (( *AorBmode &  LSBmode ) == USBmode )
				{
					*AorBmode &= ~LSBmode;
					*AorBmode |= CWmode;
					SetTimer4Bug ( wBugSpeed );
					ResetBitIOMO;
					SetBitIOSB;
					ResetIOPTT;
				}
				else if (( *AorBmode &  LSBmode ) == CWmode )
				{
					*AorBmode &= ~LSBmode;
					*AorBmode |= LSBmode;
					SetBitIOSB;
					SetIOMO;
				};
				ModeChanged = 1;
				OutputdataDirty = 1;  // Memorize changes
			};
			subfunction = 0;
			subfunctiontiming = 0;
			goto SetDefault;
		};
		if  ( UpdateDDS )
		{
			UpdateDDS = 0;
			ModeChanged = 1;
			SetAD9834value();
		};
		if ( subfunction == BUTTONRIT )
		{
			if ( subfunctiontiming < TIME_2_ACTIVATE_SUBFUNCTON )
			{
				if (( *AorBmode & RITmode ) == 0 )
				{
					*AorBmode |= RITmode;
					if (( param.Modes & BFQmode ) == AFQmode )
					{
						AD9834RIT_Afrequency = param.AD9834Afrequency;
					}
					else if (( param.Modes & BFQmode ) == BFQmode )	
					{	
						AD9834RIT_Bfrequency = param.AD9834Bfrequency;
					}
					else
						*AorBmode &= ~RITmode;
				}
				else
				{
					*AorBmode &= ~RITmode;
				};
				subfunction = 0;
				subfunctiontiming = 0;
				goto SetDefault;
			};
		};
		if ( subfunction == BUTTONABSplit )
		{
			if ( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON )
			{
				if ( bSPLITFQmode )	
				{
					bSPLITFQmode = 0;
				}
				else
				{
					bSPLITFQmode = 1;
					if (( param.Modes & BFQmode ) == BFQmode )
					{
						param.AD9834Afrequency = param.AD9834Bfrequency;
					}
					else
					{
						param.AD9834Bfrequency = param.AD9834Afrequency;
					};
				};
//				ModeChanged = 1;  // Change DDS
//				OutputdataDirty = 1;
			}
			else
			{
				if (( param.Modes & BFQmode ) == BFQmode )
				{
					param.Modes &= ~BFQmode;
					param.Modes |= AFQmode;
					AorBmode = &param.Modes;
				}
				else
				{
//					param.Modes &= ~BFQmode;
					param.Modes &= ~AFQmode;	//20100117 Tobbe
					param.Modes |= BFQmode;
					AorBmode = &param.ModesB;
				};
				if (( *AorBmode &  LSBmode ) == LSBmode )	// Added setting of I/O bits 20100117 Tobbe
				{
					SetBitIOSB;
					SetBitIOMO;
				};
				if (( *AorBmode &  LSBmode ) == CWmode )
				{
					SetBitIOSB;
					ResetBitIOMO;
				};
				if (( *AorBmode &  LSBmode ) == USBmode )
				{
					ResetBitIOSB;
					SetBitIOMO;
				};
				ModeChanged = 1;
				OutputdataDirty = 1;  // Memorize changes
				SetAD9834value();
			};
			subfunction = 0;
			subfunctiontiming = 0;
//			goto SetDefault;
		};
		if ( subfunction == BUTTONBdw )
		{
			if ( subfunctiontiming < TIME_2_ACTIVATE_SUBFUNCTON )
			{		
				SaveBandData ( Band, AddAD9834Afrequency, sizeof ( struct Parameters ));
				if ( Band > B160 )
				{
					Band--;
				}
				else
				{
					Band = Transverter;
				};
				SetBandData ( Band );
				ModeChanged = 1;  // Change DDS
				OutputdataDirty = 1; //Set/reset IO-bits
			};
			subfunction = 0;
			subfunctiontiming = 0;
			goto SetDefault;
		};
		if ( subfunction == BUTTONStep )
		{
			if ( subfunctiontiming < TIME_2_ACTIVATE_SUBFUNCTON )
			{
				memset ( (BYTE*)LastDisplayBuffer, ' ', CHAR_ON_DISPLAY );
				if ( param.FQstepLen == SMALL_FQ_STEPS )
				{
					param.FQstepLen = MEDIUM_FQ_STEPS;
				}
				else
				{
					param.FQstepLen = SMALL_FQ_STEPS;
				};
			};
			subfunction = 0;
			subfunctiontiming = 0;
			goto SetDefault;
		};
		if ( subfunction == BUTTONBup )
		{
			if ( subfunctiontiming > TIME_2_ACTIVATE_SUBFUNCTON )
			{
				send2EEPROM ( AddBugSpeed, (BYTE*)&wBugSpeed, 1 );
			}
			else
			{
				SaveBandData ( Band, AddAD9834Afrequency, sizeof ( struct Parameters ));
				if ( Band < Transverter )
				{
					Band++;
				}
				else
				{
					Band = B160;
				};
				SetBandData ( Band );
				ModeChanged = 1;  // Change DDS
				OutputdataDirty = 1; //Set/reset IO-bits
			};
			subfunctiontiming = 0;
			subfunction = 0;
			goto SetDefault;
		};
		AlreadyFound = true;			
	};
	if ( AlreadyFound )
		goto SkipMenu; //buttons not used
		
	if ( mainmode == RXmode )
	{
//		ResetIOKO;
		ResetBitIOEX; //No CW training in RX mode
		if ( CheckButton ( BUTTONFlt, NOREPEAT ) )
		{
			subfunction = BUTTONFlt;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONPre, NOREPEAT )) 
		{		
			subfunction = BUTTONPre;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONMd, NOREPEAT )) 
		{		
			subfunction = BUTTONMd;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONRIT, NOREPEAT )) 
		{		
			subfunction = BUTTONRIT;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONABSplit, NOREPEAT )) 
		{		
			subfunction = BUTTONABSplit;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONBdw, NOREPEAT )) 
		{		
			subfunction = BUTTONBdw;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONStep, NOREPEAT )) 
		{		
			subfunction = BUTTONStep;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};

		if ( CheckButton ( BUTTONBup, NOREPEAT ))
		{		
			subfunction = BUTTONBup;
			subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
			goto SetDefault;
		};
SetDefault:
		AlreadyFound = true;	
		ValueChanged = false;
		goto SkipMenu;
	}
	else if ( mainmode == TXmode )
	{
		ResetBitIOEX;  //No CW training in TX mode
		if ( CheckButton4repeat ( BUTTONPre, BUGSPEEDREPEAT, BUGSPEEDREPEATINTERVALL ))
		{		
			goto SkipMenu;
//			goto SetDefault;
		};
		if ( CheckButton4repeat ( BUTTONRIT, BUGSPEEDREPEAT, BUGSPEEDREPEATINTERVALL )) 
		{		
			goto SkipMenu;
//			goto SetDefault;
		};
		if (LowPowerSet) //To enable tuningpower setting during transmit
		{
			if (CheckButton( BUTTONBdw, NOREPEAT))
			{
				subfunction = BUTTONBdw;
				subfunctiontiming = 1; //Start counting 5 ms periods for subfunction
				goto SetDefault;	
			};
		};
	}
	else if ( mainmode == MENUMODE )
	{
		if ( CheckButton ( BUTTONMd, NOREPEAT )) 
		{		
			mainmode = RXmode;
			//Clean display buffer for normal entry
			for ( idx = 0; idx < CHAR_ON_DISPLAY; idx++ )
				DisplayBuffer[ idx ] = ' ';
			AlreadyFound = true;
			LastMenuRow = menuPosRow;
			menuPosCol = 0;
			menuPosRow = 0;
			if ( IsIOEX_set )
					ResetIOEX;
			ControlChanged = 0;
			EncoderCW_count 	= MAXENCODERCOUNT; //So fq is not changing!
			EncoderCCW_count 	= MAXENCODERCOUNT;
		};
		if ( CheckButton ( BUTTONFlt, NOREPEAT ))
		{		
			AlreadyFound = true;
			if ( menuPosCol > 0 )
			{
				menuPosCol--;
			};
		};
		if ( CheckButton ( BUTTONPre, NOREPEAT ))
		{		
			AlreadyFound = true;
			menuPosCol = 0;
			memset ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], ' ', CHAR_ON_ONE_LINE ); 
			memcpy ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], 
											&cMenu[0], strlen ( cMenu ));
			if ( menuPosRow > 1 )
			{
				menuPosRow--;
			}
			else
				menuPosRow = LASTUSEDROWVALUE;
		};
		if ( CheckButton ( BUTTONRIT, NOREPEAT ))
		{		
			AlreadyFound = true;
			memset ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], ' ', CHAR_ON_ONE_LINE ); 
			memcpy ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], 
									&cMenu[0], strlen ( cMenu ));
			if ( menuPosCol == 0 )
			{
				if (( LASTUSEDROWVALUE ) > menuPosRow )
				{
					menuPosRow++;
//					menuPosCol = 0;
				}
				else
				{
					menuPosRow = 1;
//					menuPosCol = 0;
				};
			};
		};
		if ( CheckButton ( BUTTONABSplit, NOREPEAT ))
		{
			AlreadyFound = true;
			if ( menuPosCol < ( SUBLEVELS - 1 ))
			{
				menuPosCol++;
			};
		};
		if ( CheckButton ( BUTTONBup, NOREPEAT )) 
		{		
			if ( DecimalPosition > 0 )
				DecimalPosition--;
			memset ( (BYTE*)LastDisplayBuffer, ' ', CHAR_ON_DISPLAY );
			goto SetDefault;
		};
		if ( CheckButton ( BUTTONBdw, NOREPEAT )) 
		{		
			if ( DecimalPosition < MAXDDSREFDECIMALS - 1 )
				DecimalPosition++;
			memset ( (BYTE*)LastDisplayBuffer, ' ', CHAR_ON_DISPLAY );
			goto SetDefault;
		};
		
	};	
SkipMenu:;
//20081207    
  if (( menuPosCol == 0 ) && ( mainmode == MENUMODE ))
  {
      if ( CheckEncoderBYTE ( (BYTE*)&bMenuSteps, 4, 1 )) 
      {
				memset ( (char*)&DisplayBuffer[CHAR_ON_ONE_LINE], ' ', CHAR_ON_ONE_LINE ); 
        if ( bMenuSteps < 2  )
        {
          if ( menuPosRow > 1 )
            menuPosRow--;
          else
            menuPosRow = LASTUSEDROWVALUE - 1;
        };
        if ( bMenuSteps > 2 )
				{
					if (( LASTUSEDROWVALUE ) > menuPosRow )
					{
						menuPosRow++;
					}
					else
					{
						menuPosRow = 1;
					};
				};
      };
      if ( KnobGoingActive )
      {
        KnobGoingActive = false;
        if ( menuPosCol < ( SUBLEVELS - 1 ))
        {
          menuPosCol++;
        };
      };
  };
//20081207    
	CursorPosition = CHAR_ON_DISPLAY;
	
	if (( *(FunctionPointer [ menuPosRow ][ menuPosCol ]) > 0 ) &&
			( mainmode == MENUMODE ))
			(FunctionPointer [ menuPosRow ][ menuPosCol ])();
			
};


//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetDisplayMode ( void )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSetDisplayMode[0], strlen ( cSetDisplayMode ));
	
	if ( DisplayMode == DisplayOff )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
							&cDisplayOff[0], strlen ( cDisplayOff ));
	};
	if ( DisplayMode == DisplayOn )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
										&cDisplayOn[0], strlen ( cDisplayOn ));
	};
	if ( DisplayMode == DisplayAuto )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
						&cDisplayAuto[0], strlen ( cDisplayAuto ));
	};
	if ( menuPosCol == 0 ) 
		return;
	
	if ( CheckEncoderBYTE ( (BYTE*)&DisplayMode, 2, 1 )) 
	{
	};
	if ( KnobGoingActive )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		send2EEPROM ( AddDisplayMode, (BYTE*)&DisplayMode, 1 );
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
		wDisplayBackLiteTime = 0;
		
//		if ( DisplayMode == DisplayOff ) //Not necessary??? 20091128
//			SwitchOFFbacklite;		
		if ( DisplayMode == DisplayOn )
			SwitchONbacklite;		
		if ( DisplayMode == DisplayAuto )
		{
			wDisplayBackLiteTime = TEN_SECONDS_DELAY;
			SwitchONbacklite;		
		};
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void 	ShowRealBugSpeed ( void )
{
	UINT  helpCount = 600000000/1012;
	WORD	wCalculatedBugSpeed;
	
	wCalculatedBugSpeed = helpCount / wBugCountValue;
	if (( wCalculatedBugSpeed % 10 ) > 4 )
		wCalculatedBugSpeed += 5;
	wCalculatedBugSpeed /= 10;

	memset ( (char*)&DisplayBuffer [ POS_SMETER ], ' ', 
										CHAR_ON_ONE_LINE - POS_SMETER );
	MakeWord2string ( (char*)DisplayBuffer, 
		wCalculatedBugSpeed, POS_SMETER );

};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetDdsRefFrequency ( void )
{
	BYTE	idx;
	BYTE	jdx;

	BYTE	ChangedData = 0;
	BYTE	offset;
	UINT	StepSize = 10;
	int		FqDiff;
  if ( menuPosCol == 0 ) 
  {
    memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_ONE_LINE );
    memcpy ( (char*)&DisplayBuffer [ 0 ], &cSetDdsRefFrequency [0],
        strlen ( cSetDdsRefFrequency ));
    readFromEEPROM ( AddAD9834RefFrequency, (BYTE*)&AD9834RefFrequency, 4 );
    if (( AD9834RefFrequency < 98000000 ) || ( AD9834RefFrequency > 102000000 ))
      AD9834RefFrequency = 100000000;
    DecimalPosition = 0;
  };
  memset ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], ' ', CHAR_ON_ONE_LINE );//20081229 

	for ( idx = 0; idx < DecimalPosition; idx++ )
		StepSize *= 10;
  
  // For line 2 on display
  NumberOfDigits = int2stringWsign ( (char*)NumberBuffer, AD9834RefFrequency );
  offset = FQ_LASTPOSITION - NumberOfDigits + CHAR_ON_ONE_LINE - 1;
  jdx = offset;
  for ( idx = 0; idx < (NumberOfDigits - 1 ); idx++ ) //to - 1 20071102
  {
    if ( jdx == ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 3 )) 
    {
      DisplayBuffer [ jdx++ ] = '.';
    };
    if ( jdx == ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 7 )) 
    {
      DisplayBuffer [ jdx++ ] = '.';
    };
    DisplayBuffer [ jdx++ ] = NumberBuffer [ idx ];
  };
	DisplayBuffer [ jdx++ ] = '0';
	DisplayBuffer [ jdx++ ] = ' ';
	DisplayBuffer [ jdx++ ] = 'H';
	DisplayBuffer [ jdx   ] = 'z';

  if ( DecimalPosition == 0 )
    CursorPosition = ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 1 );
  else if ( DecimalPosition == 1 )
    CursorPosition = ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 2 );
  else if ( DecimalPosition == 2 )
    CursorPosition = ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 4 );
  else if ( DecimalPosition == 3 )
    CursorPosition = ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 5 );
  else if ( DecimalPosition == 4 )
    CursorPosition = ( FQ_LASTPOSITION + CHAR_ON_ONE_LINE - 7 );
  	
	ChangedData = CheckEncoderLONGWORD ( (UINT*)&AD9834RefFrequency, StepSize  );

	if ( KnobGoingActive )
	{
		KnobGoingActive = false;
		AlreadyFound = true;
		for ( idx = 0; idx < CHAR_ON_DISPLAY; idx++ )
		{
			DisplayBuffer [ idx ] = ' ';
		};
		CalculateDDSfractions ( AD9834RefFrequency );
		send2EEPROM ( AddAD9834RefFrequency, (BYTE*)&AD9834RefFrequency, 4 );
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};


//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetIambicMode ( void )
{
	if ( bIambicMode > StraightKey )
	{
	  bIambicMode = NoIambicMode;
		send2EEPROM ( AddIambicMode, (BYTE*)&bIambicMode, 1 );
	};
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cIambic[0], strlen ( cIambic ));
	
	if ( bIambicMode == NoIambicMode )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cIambicOff[0], strlen ( cIambicOff ));
	};
	if ( bIambicMode == IambicModeA )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cIambicA[0], strlen ( cIambicA ));
	};
	if ( bIambicMode == IambicModeB )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cIambicB[0], strlen ( cIambicB ));
	};
	if ( bIambicMode == AccukeyMode )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cAccuKey[0], strlen ( cAccuKey ));
	};
	if ( bIambicMode == StraightKey )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cStraightKey[0], strlen ( cStraightKey ));
	};
	
//20081207	->	
		if ( menuPosCol == 0 ) 
			return;
//20081207	<-	
	if ( CheckEncoderBYTE ( (BYTE*)&bIambicMode, sizeof ( enum IambicMode ), 1 ))
	{
	  //20080929		if ( bIambicMode == StraightKey )
		{
				/* Enable button interrupts generation through the PIO controller */
		  //20080929		  AT91C_BASE_PIOA->PIO_IER = ( Bug0 | Bug1 );	
		}
		//20080929		else
		{
		  //20080929		  AT91C_BASE_PIOA->PIO_IDR = ( Bug0 | Bug1 );	
		};
	};
	if ( KnobGoingActive )
	{
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		send2EEPROM ( AddIambicMode, (BYTE*)&bIambicMode, 1 );
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


void ShowBoardVoltage ( UINT VoltageValue, BYTE position )
{
	BYTE		idx;
	UINT		voltage;
	
	voltage = VoltageValue / 100;

	MakeWord2string ( (char*)&DisplayBuffer [ position ], (WORD)voltage, 0 );
	DisplayBuffer [ position + 3 ] = DisplayBuffer [ position + 2 ]; 
	DisplayBuffer [ position + 2 ] = DisplayBuffer [ position + 1 ]; 
	DisplayBuffer [ position + 1 ] = ',';
	DisplayBuffer [ position + 4 ] = 'V';
};

//++++++++++++++++++++++++++++++++++++++++++++

void SetBugConnect ( void )
{
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cBugConnect[0], strlen ( cBugConnect ));
	memcpy ( (char*)&DisplayBuffer[ CHAR_ON_ONE_LINE ], 
									&cBugConnectMode[0], 
									strlen ( cBugConnectMode ) );
	
	if ( BugConnection > 1 )
		BugConnection = 0;
//20081207	->	
	if ( BugConnection == 0 )
	{
		DisplayBuffer[ strlen ( cBugConnectMode ) + CHAR_ON_ONE_LINE + 1 ] = '0'; 
	}
	else
	{
		DisplayBuffer[ strlen ( cBugConnectMode ) + CHAR_ON_ONE_LINE + 1 ] = '1'; 
	};
		if ( menuPosCol == 0 ) 
			return;
//20081207	<-	
	if ( CheckEncoderBYTE ( (BYTE*)&BugConnection, 1, 1 )) 
	{
	};
	if ( KnobGoingActive )
	{
		send2EEPROM ( AddBugConnectionMode, (BYTE*)&BugConnection, 1 );
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		menuPosCol = 0;
		if ( menuPosRow > 1 )
			menuPosRow--;
	};
};

//+++++++++++++++++++++++++++++++++++++++++++++++++

void SetButtonSound ( void )
{
	WORD	value;
	
	if ( wButtonSoundLen > MAX_LENGTH_OF_BUTTON_TONE )
		wButtonSoundLen = 0;
	memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cSetButtonSound [0], 
																			strlen ( cSetButtonSound ));
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], 
																		&cButtonSound[0], 
																		strlen ( cButtonSound ) );
	
	if ( wButtonSoundLen < MIN_LENGTH_OF_BUTTON_TONE )
	{
		memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE + strlen ( cButtonSound )], 
																			" OFF", 
																			strlen ( " OFF" ) );
	}
	else
	{
		MakeWord2string ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE + 
																						strlen ( cButtonSound ) ], 
																						(	wButtonSoundLen / 10 ), 0 );
	};
	if ( menuPosCol == 0 ) 
		return;
	if ( CheckEncoderWORD ( (WORD*)&wButtonSoundLen, MAX_LENGTH_OF_BUTTON_TONE, 50 )) 
	{
	};
	if ( KnobGoingActive )
	{
		send2EEPROM ( AddwButtonSoundLen, (BYTE*)&wButtonSoundLen, 2 );
		memset ( (char*)&DisplayBuffer [ 0 ], ' ', CHAR_ON_DISPLAY );
		KnobGoingActive = false;
		AlreadyFound = true;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


void Voltages ( void )
{
	memcpy ( (char*)&DisplayBuffer [ 0 ], &cVoltages [0],
			strlen ( cVoltages ));
	memcpy ( (char*)&DisplayBuffer [ CHAR_ON_ONE_LINE ], &cVoltages2 [0],
			strlen ( cVoltages2 ));
	ShowBoardVoltage (( GetAT91SAM_ADCvalue ( AdcChannel9volt ) * Calc_9volt ), 
			POS_9VOLT );
	ShowBoardVoltage (( GetAT91SAM_ADCvalue ( AdcChannel5volt ) * Calc_5volt ), 
			POS_5VOLT );
	ShowBoardVoltage (( GetAT91SAM_ADCvalue ( AdcChannel3_3volt ) * Calc_3_3volt ), 
			POS_3_3VOLT );
	if ( KnobGoingActive )
	{
		KnobGoingActive = false;
		AlreadyFound = true;
		if ( menuPosRow > 1 )
			menuPosRow--;
		menuPosCol = 0;
	};
};


//+++++++++++++++++++++++++++++++++++++++++++++++++
